PROGRAM XFEM
!-------------------------------------------------------------------------
! Program XFEM    Hydro-Mechanical coupled analysis of an elastic solid 
!                 using 4-node rectangular quadrilaterals with fractures
!                 demonstrated by the level set function and XFEM
!-------------------------------------------------------------------------
 USE main
 USE geom
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 INTEGER::fixed_freedoms,i,ii,iel,k,loaded_nodes,ndim=2,ndof,nels,neq,nip,&
   nn,nod,nodof=2,nprops=2,np_types,nr,nst=3,nxe,nye,nlen,iter,dof,bdcd
 REAL(iwp)::det,one=1.0_iwp,penalty=1.0e20_iwp,zero=0.0_iwp,height,width, &
   lxe,lye,frx1,fry1,frx2,fry2,radi,dfcr,detmt
 CHARACTER(LEN=15)::element,dir,type_2d,argv,compa
!-----------------------dynamic arrays------------------------------------
 INTEGER,ALLOCATABLE::etype(:),g(:),g_g(:,:),g_num(:,:),kdiag(:),nf(:,:), &
   no(:),node(:),num(:),sense(:),loca(:),connec(:,:),nzph(:),nzps(:),     &
   enrelem(:),iielem(:),frdof(:)
 REAL(iwp),ALLOCATABLE::bee(:,:),coord(:,:),dee(:,:),der(:,:),deriv(:,:), &
   eld(:),fun(:),gc(:),g_coord(:,:),jac(:,:),km(:,:),kv(:),loads(:),      &
   points(:,:),prop(:,:),sigma(:),value(:),weights(:),x_coords(:),        &
   y_coords(:),nodes(:,:),xyz(:,:),frt(:,:),inc(:,:),void(:,:),gphi(:,:), &
   gpsi(:,:),chi(:,:),zeta(:,:),omega(:,:),disp(:,:),phdof(:),bdld(:),    &
   gforce(:,:),&
   rMAT(:,:),rRHS(:),sol(:,:),gdisp(:),sxx(:,:),sxy(:,:),syy(:,:),svm(:,:)
!-----------------------input and initialisation--------------------------
 CALL getname(argv,nlen)
 OPEN(10,FILE=argv(1:nlen)//'.dat')
 READ(10,*)type_2d,element,nod,dir,nxe,nye,nip,np_types
 CALL mesh_size(element,nod,nels,nn,nxe,nye)
 ndof=nod*nodof
 IF(type_2d=='axisymmetric')nst=4
 ALLOCATE(nf(nodof,nn),points(nip,ndim),g(ndof),g_coord(ndim,nn),fun(nod),&
   coord(nod,ndim),jac(ndim,ndim),g_num(nod,nels),der(ndim,nod),          &
   deriv(ndim,nod),bee(nst,ndof),km(ndof,ndof),eld(ndof),weights(nip),    &
   g_g(ndof,nels),prop(nprops,np_types),num(nod),x_coords(nxe+1),         &
   y_coords(nye+1),etype(nels),gc(ndim),dee(nst,nst),sigma(nst),          &
   nodes(nn,31),connec(nxe*nye,5),xyz(nn,3),frt(2,2),inc(1,1),void(1,1),  &
   bdld(3))
 READ(10,*)prop
 READ(10,*)height,width
 READ(10,*)frx1,fry1,frx2,fry2,radi
 nf=1
 READ(10,*)nr,(k,nf(:,k),i=1,nr)
 CALL formnf(nf)
 neq=MAXVAL(nf)
 ALLOCATE(loads(0:neq),kdiag(neq))
!-----------------------loop the elements to find global arrays sizes-----
 kdiag=0
 iter=1
 ii=1
 !
	CALL coord_connec(width,height,frx1,frx2,fry1,fry2,nxe,nye,lxe,lye,frt,&
							nodes,connec,xyz)
	ALLOCATE(phdof(1))
	phdof=0
	!
	CALL levelset(iter,nxe,nye,lxe,lye,radi,connec,frt,nodes,gphi,gpsi,chi, &
					zeta,inc,void,omega)
	!
	CALL calc_dof(nodes,dof,disp)
	CALL enriched_elem(ii,phdof,connec,nodes,enrelem,iielem)
	!
	CALL stiff_mat(omega,chi,zeta,connec,frt,lxe,nxe,nye,prop,nodes,gphi,gpsi,xyz,&
					dof,iter,enrelem,iielem,km)
	!
	bdcd=3
	bdld=(/1,0,1000/)
	CALL force_vector(dof,nxe,nye,lxe,lye,bdld,connec,gforce)
	CALL boundary_cond(dof,bdcd,chi,connec,nxe,nye,nodes,void,frdof)
	ALLOCATE(rMAT(SIZE(frdof),SIZE(frdof)),rRHS(SIZE(frdof)),gdisp(dof))
	gdisp=0
	!
	rMAT=km(frdof,frdof)
	rRHS=gforce(frdof,1)
	!
	CALL Linear_Solver(rMAT,rRHS,SIZE(frdof),1,sol,detmt)
	gdisp(frdof)=sol(:,1)
	
	CALL elem_stress(omega,gdisp,chi,zeta,connec,frt,lxe,nxe,nye,prop,nodes,&
					gphi,gpsi,xyz,sxx,sxy,syy,svm)
	
	OPEN(UNIT=101,FILE="disp.txt")
	DO i=1,SIZE(gdisp)
		WRITE(101,*)i,gdisp(i)
	END DO
	!
 STOP
 ! 
END PROGRAM XFEM