SUBROUTINE branch_nodes(frt,nodes,gphi,xyz)
!
! This function assigns nodes enriched with the crack tip enrichment function values 
! associated with the near tip asymptotic displacement field.
!
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 REAL(iwp),INTENT(INOUT)::nodes(:,:)
 REAL(iwp),INTENT(IN)::frt(:,:),gphi(:,:),xyz(:,:)
 REAL(iwp)::xn,yn,rt,tt,rr,theta
 REAL(iwp),ALLOCATABLE::xft(:),yft(:),xx(:),yy(:)
 INTEGER::iseg,nft,inode
 !
 iseg=SIZE(frt,1)
 nft=SIZE(gphi,2)
 !
 IF (nft==1) THEN
	ALLOCATE(xft(1),yft(1),xx(1),yy(1))
	xft=frt(iseg,1)
	yft=frt(iseg,2)
 ELSE IF (nft==2) THEN
	ALLOCATE(xft(2),yft(2),xx(2),yy(2))
	xft=(/frt(iseg,1),frt(1,1)/)
	yft=(/frt(iseg,2),frt(1,2)/)
 END IF
 !
 DO inode=1,SIZE(nodes,1)
	IF (nodes(inode,4)/=0) THEN
		xn=xyz(inode,2)
		yn=xyz(inode,3)
		xx=xn-xft
		yy=yn-yft
		DO i=1,SIZE(xx)
			rt=SQRT(xx(i)**2+yy(i)**2)
			tt=ATAN2(yy(i),xx(i))
			IF (i==1) THEN
				rr=rt
				theta=tt
			ELSE IF (i>1) THEN
				IF (rt<rr) THEN
					rr=rt
					theta=tt
				END IF
			END IF
		END DO
		nodes(inode,5)  = SQRT(rr)*SIN(theta/2.0)
		nodes(inode,7)  = SQRT(rr)*COS(theta/2.0)
		nodes(inode,9)  = SQRT(rr)*SIN(theta/2.0)*SIN(theta)
		nodes(inode,11) = SQRT(rr)*COS(theta/2.0)*SIN(theta)
	END IF
 END DO
 DEALLOCATE(xft,yft,xx,yy)
 !
 RETURN
END SUBROUTINE branch_nodes