SUBROUTINE levelset(iter,nxe,nye,lxe,lye,radi,connec,frt,nodes,gphi,gpsi, &
                    chi,zeta,inc,void,omega)
!
! This subroutine applies the level set function to determine the sigularity
! (fracture,inclusion,void) within the domain.
!
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 REAL(iwp),INTENT(IN)::lxe,lye,radi,frt(:,:),inc(:,:),void(:,:)
 REAL(iwp)::sdzr,zero=0.0,one=1.0,two=2.0,xc,yc,rc,xi,yi,ci,zi,ll,xx,yy,cc
 REAL(iwp)::dfseg,radius,xfrt,yfrt,xn,yn,dfcr,x1,y1,x2,y2,mm,bb,xo,yo,dmin,psi
 REAL(iwp)::nodes(:,:),ccs(2,2),zz(1,4),xy(2,1),xxyy(1,2),xyloc(1,2),phie(4),psie(4)
 REAL(iwp),ALLOCATABLE::temp(:),phi(:),t(:,:),disc(:,:),dist(:),disr(:),sine(:)
 REAL(iwp),ALLOCATABLE,INTENT(OUT)::gphi(:,:),gpsi(:,:),chi(:,:),zeta(:,:),omega(:,:)
 INTEGER,INTENT(IN)::iter,nxe,nye,connec(:,:)
 INTEGER::i,npt,nelem,ielem,in,nn,inode,incnode,ninc,iinc
 INTEGER::nvoid,ivoid,nft,ift,iseg,cnode,celem,ind,gn
 INTEGER,ALLOCATABLE::domain(:),ind1(:),ctelem(:),helem(:),c1(:),c2(:),c3(:),c4(:)
 INTEGER,ALLOCATABLE::ctnodes(:),hnodes(:),bmnodes(:),tempct(:),temph(:)
 CHARACTER(LEN=15)::compa
 !
 nelem=nxe*nye                                ! number of elements
 nn=(nxe+1)*(nye+1)                           ! number of nodes
 npt=SIZE(frt,1)                              ! number of single fractures
 !
 IF (lxe>=lye) THEN                           ! singularity dominate zone size
	IF (radi==0.0) THEN 
		sdzr=(SQRT(two)+0.005)*lxe
	ELSE 
		sdzr=radi*lxe
	END IF
 ELSE
	IF (radi==0.0) THEN 
		sdzr=(SQRT(two)+0.005)*lye
	ELSE 
		sdzr=radi*lye
	END IF  
 END IF
 !
 IF (iter==1) THEN
	incnode=1
 ELSE
	incnode=1+MAXVAL(nodes(:,2))-nn
 END IF 
 !
 IF (iter==1) THEN
	IF (SIZE(inc,2)>=3) THEN
		IF (SIZE(inc,2)==3) THEN
			ninc=SIZE(inc,1)
			ALLOCATE(zeta(ninc,nn))
			zeta=0
			DO iinc=1,ninc
				xc=inc(iinc,1)
				yc=inc(iinc,2)
				rc=inc(iinc,3)
				DO inode=1,nn
					xi=xyz(inode,2)
					yi=xyz(inode,3)
					zi=SQRT((xi-xc)**2+(yi-yc)**2)-rc
					IF (ABS(zi)<1E-6) THEN
						zeta(iinc,inode)=0
					ELSE
						zeta(iinc,inode)=zi
					END IF
				END DO
			END DO
		ELSE
			ALLOCATE(zeta(1,nn))
			xo=inc(1,1)
			yo=inc(1,2)
			x1=inc(1,3)
			y1=inc(1,4)
			ll=SQRT((x1-xo)**2+(y1-yo)**2)
			DO inode=1,nn
				xx=xyz(inode,2)
				yy=xyz(inode,3)
				cc=(yo-y1)*xx+(x1-xo)*yy+(xo*y1-x1*yo)
				zeta(1,inode)=cc/ll
			END DO
		END IF
		!!!zeta=MINVAL(zeta,1)
		DO ielem=1,nelem
			zz(1,1)=zeta(1,connec(ielem,2))
			zz(1,2)=zeta(1,connec(ielem,3))
			zz(1,3)=zeta(1,connec(ielem,4))
			zz(1,4)=zeta(1,connec(ielem,5))
			IF (MAXVAL(zz)*MINVAL(zz)<0) THEN
				DO inode=2,5
					IF (nodes(connec(ielem,inode),30)==0) THEN
						nodes(connec(ielem,inode),30)=nn+incnode
						nodes(connec(ielem,inode),31)=zz(1,inode-1)
					END IF
				END DO
			END IF
		END DO
	ELSE
		ALLOCATE(zeta(1,nn))
		zeta=1
	END IF
	!
	IF (SIZE(void,2)==3) THEN
		nvoid=SIZE(void,1)
		ALLOCATE(chi(nvoid,nn))
		DO ivoid=1,nvoid
			xc=void(ivoid,1)
			yc=void(ivoid,2)
			rc=void(ivoid,3)
			DO inode=1,nn
				xi=xyz(inode,2)
				yi=xyz(inode,3)
				ci=SQRT((xi-xc)**2+(yi-yc)**2)-rc
				IF (ABS(ci)<1E-6) THEN
					chi(ivoid,inode)=0
				ELSE
					chi(ivoid,inode)=ci
				END IF
			END DO
		END DO
		!!!chi=MINVAL(chi,1)
	ELSE
		ALLOCATE(chi(1,nn))
		chi=0
	END IF
 END IF
 !
 IF (frt(1,1)/=frt(2,1)		.OR.	frt(1,2)/=frt(2,2))  THEN
	nft=2
	IF (frt(1,1)==0        	.OR.	frx1==nxe*lxe)       THEN
		nft=nft-1
	ELSE IF (frt(npt,1)==0 	.OR.	frt(npt,1)==nxe*lxe) THEN
		nft=nft-1
	ELSE IF (frt(1,2)==0	.OR.	frt(1,2)==nye*lye)   THEN
		nft=nft-1
	ELSE IF (frt(npt,2)==0 	.OR.	frt(npt,2)==nye*lye) THEN
		nft=nft-1
	END IF
	!
	IF (nft==2) THEN
		ALLOCATE(disc(1,2),t(1,2),phi(2),omega(1,2))
		disc(1,:)=frt(npt,:)-frt(npt-1,:)
		omega(1,1)=ATAN2(disc(1,2),disc(1,1))
		disc(1,:)=frt(1,:)-frt(2,:)
		omega(1,2)=ATAN2(disc(1,2),disc(1,1))
	ELSE IF (nft==1) THEN
		ALLOCATE(disc(1,2),t(1,2),phi(1),omega(1,1))
		disc(1,:)=frt(npt,:)-frt(npt-1,:)
		omega=ATAN2(disc(1,2),disc(1,1))
	END IF
	phi=0
	!
	ALLOCATE(dist(npt-1),sine(npt-1))
	dist=0
	sine=0
	ALLOCATE(disr(nn),temp(nn))
	disr=0
	temp=0
	ALLOCATE(gphi(nn,nft))
	gphi=0.0
	!
	IF (iter==1) THEN
		ALLOCATE(gpsi(nn,1))
		gpsi=0.0
	END IF
	!
	DO ift=1,nft
		IF (ift==1) THEN
			dfseg=SQRT((frt(npt-1,1)-frt(npt,1))**2+(frt(npt-1,2)-frt(npt,2))**2)
			radius=dfseg+sdzr
			xfrt=frt(npt,1)
			yfrt=frt(npt,2)
			ccs(1,:)=(/COS(omega(1,1)),SIN(omega(1,1))/)
			ccs(2,:)=(/-SIN(omega(1,1)),COS(omega(1,1))/)
		ELSE IF (ift==2) THEN
			dfseg=SQRT((frt(1,1)-frt(2,1))**2+(frt(1,2)-frt(2,2))**2)
			radius=dfseg+sdzr
			xfrt=frt(1,1)
			yfrt=frt(1,2)
			ccs(1,:)=(/COS(omega(1,2)),SIN(omega(1,2))/)
			ccs(2,:)=(/-SIN(omega(1,2)),COS(omega(1,2))/)
		END IF
		!
		disr=0.0
		DO in=1,nn
			xn=xyz(in,2)
			yn=xyz(in,3)
			xx=xn-xfrt
			yy=yn-yfrt
			xyloc(1,1)=xx
			xyloc(1,2)=yy
			xy=MATMUL(ccs,TRANSPOSE(xyloc))
			disr(in)=SQRT(xy(2,1)**2+xy(1,1)**2)
		END DO
		!
		temp=disr-radius
		compa='le'
		dfcr=0.0
		CALL find_loc(temp,compa,dfcr,domain)
		!
		DO inode=1,SIZE(domain)
			cnode=domain(inode)
			xx=xyz(cnode,2)
			yy=xyz(cnode,3)
			!
			disc(1,:)=frt(npt,:)-frt(npt-1,:)
			t=disc/SQRT(disc(1,1)**2+disc(1,2)**2)
			phi(1)=(xx-frt(npt,1))*t(1,1)+(yy-frt(npt,2))*t(1,2)
			!
			IF (phi(1)==0) THEN
				phi(1)=1E-6
			END IF
			!
			IF (nft==2) THEN
				disc(1,:)=frt(1,:)-frt(2,:)
				t=disc/SQRT(disc(1,1)**2+disc(1,2)**2)
				phi(2)=(xx-frt(1,1))*t(1,1)+(yy-frt(1,2))*t(1,2)
				IF (phi(2)==0) THEN
					phi(2)=1E-6
				END IF
			END IF
			!   
			IF (ift==1) THEN
				IF (ABS(phi(1))<sdzr) THEN
					dist=0
					sine=0
					DO iseg=1,(npt-1)
						x1=frt(iseg  ,1)
						y1=frt(iseg  ,2)
						x2=frt(iseg+1,1)
						y2=frt(iseg+1,2)
						!     
						mm=(y2-y1)/(x2-x1)
						IF (mm>=1E6) THEN
							mm=1E6
						END IF
						bb=y1-mm*x1
						!
						xo=(mm*yy+xx-mm*bb)/(mm**2+1)
						yo=(mm**2*yy+mm*xx+bb)/(mm**2+1)
						!
						IF (iseg/=1) THEN
							IF (xo<x1) THEN
								xo=x1
								yo=y1
							END IF
						END IF
						!
						IF (iseg/=(npt-1)) THEN
							IF (xo>x2) THEN
								xo=x2
								yo=y2
							END IF
						END IF
						!
						dist(iseg)=SQRT((xo-xx)**2+(yo-yy)**2)
						sine(iseg)=SIGN(one,(x2-x1))*SIGN(one,(yy-yo))
					END DO
					!
					dmin=MINVAL(ABS(dist))
					compa='eq'
					CALL find_loc(ABS(dist),compa,dmin,ind1)
					ind=ind1(1)
					psi=dist(ind)*sine(ind)
					IF (psi==0.0) THEN
						psi=1E-6
					END IF
					!
				IF (ABS(psi)<sdzr) THEN
					gphi(cnode,1)=phi(1)
					gpsi(cnode,1)=psi
				END IF
				!
				ELSE IF (phi(1)<0) THEN
					dist=0
					sine=0
					DO iseg=1,npt-1
						x1=frt(iseg  ,1)
						y1=frt(iseg  ,2)
						x2=frt(iseg+1,1)
						y2=frt(iseg+1,2)
						!      
						mm=(y2-y1)/(x2-x1)
						IF (mm>=1E6) THEN
							mm=1E6
						END IF
						bb=y1-mm*x1
						!
						xo=(mm*yy+xx-mm*bb)/(mm**2+1)
						yo=(mm**2*yy+mm*xx+bb)/(mm**2+1)
						!
						IF (iseg/=1) THEN
							IF (xo<x1) THEN
								xo=x1
								yo=y1
							END IF
						END IF
						!
						IF (iseg/=(npt-1)) THEN
							IF (xo>x2) THEN
								xo=x2
								yo=y2
							END IF
						END IF
						!
						dist(iseg)=SQRT((xo-xx)**2+(yo-yy)**2)
						sine(iseg)=SIGN(one,(x2-x1))*SIGN(one,(yy-yo))
					END DO
					!
					dmin=MINVAL(ABS(dist))
					compa='eq'
					CALL find_loc(ABS(dist),compa,dmin,ind1)
					ind=ind1(1)
					psi=dist(ind)*sine(ind)
					IF (psi==0) THEN
						psi=1E-6
					END IF
					!
					IF (ABS(psi)<sdzr) THEN
						gpsi(cnode,1)=psi
					END IF
				END IF
			END IF
			!
			IF (ift==2) THEN
				IF (ABS(phi(2))<sdzr) THEN
					dist=0
					sine=0
					DO iseg=1,npt-1
						x1=frt(iseg  ,1)
						y1=frt(iseg  ,2)
						x2=frt(iseg+1,1)
						y2=frt(iseg+1,2)
						!  
						mm=(y2-y1)/(x2-x1)
						IF (mm>=1E6) THEN
							mm=1E6
						END IF
						bb=y1-mm*x1
						!
						xo=(mm*yy+xx-mm*bb)/(mm**2+1)
						yo=(mm**2*yy+mm*xx+bb)/(mm**2+1)
						!
						IF (iseg/=1) THEN
							IF (xo<x1) THEN
								xo=x1
								yo=y1
							END IF
						END IF
						!
						IF (iseg/=(npt-1)) THEN
							IF (xo>x2) THEN
								xo=x2
								yo=y2
							END IF
						END IF
						!
						dist(iseg)=SQRT((xo-xx)**2+(yo-yy)**2)
						sine(iseg)=SIGN(one,(x2-x1))*SIGN(one,(yy-yo))
					END DO
					!
					dmin=MINVAL(ABS(dist))
					compa='eq'
					CALL find_loc(ABS(dist),compa,dmin,ind1)
					ind=ind1(1)
					psi=dist(ind)*sine(ind)
					IF (psi==0) THEN
						psi=1E-6
					END IF
					!
					IF (ABS(psi)<sdzr) THEN
						IF (gphi(cnode,1)/=0) THEN
							gphi(cnode,1)=phi(2)
							gpsi(cnode,1)=psi
						END IF
					ELSE
						gphi(cnode,1)=phi(2)
						gpsi(cnode,1)=psi
					END IF
					!
				ELSE IF (phi(2)<0) THEN
					IF (phi(1)<0) THEN
						dist=0
						sine=0
						DO iseg=1,npt-1
							x1=frt(iseg  ,1)
							y1=frt(iseg  ,2)
							x2=frt(iseg+1,1)
							y2=frt(iseg+1,2)
							!      
							mm=(y2-y1)/(x2-x1)
							IF (mm>=1E6) THEN
								mm=1E6
							END IF
							bb=y1-mm*x1
							!
							xo=(mm*yy+xx-mm*bb)/(mm**2+1)
							yo=(mm**2*yy+mm*xx+bb)/(mm**2+1)
							!
							IF (iseg/=1) THEN
								IF (xo<x1) THEN
									xo=x1
									yo=y1
								END IF
							END IF
							!
							IF (iseg/=(npt-1)) THEN
								IF (xo>x2) THEN
									xo=x2
									yo=y2
								END IF
							END IF
							!
							dist(iseg)=SQRT((xo-xx)**2+(yo-yy)**2)
							sine(iseg)=SIGN(one,(x2-x1))*SIGN(one,(yy-yo))
						END DO
						!
						dmin=MINVAL(ABS(dist))
						compa='eq'
						CALL find_loc(ABS(dist),compa,dmin,ind1)
						ind=ind1(1)
						psi=dist(ind)*sine(ind)
						IF (psi==0) THEN
							psi=1E-6
						END IF
						!
						IF (ABS(psi)<sdzr) THEN
							gpsi(cnode,1)=psi
						END IF
					END IF
				END IF
			END IF
		END DO
	END DO
	ALLOCATE(ctnodes(nn))
	ctnodes=0
	ALLOCATE(bmnodes(nn))
	bmnodes=0
	dmin=zero
	compa='ne'
	CALL find_loc(gphi(:,1),compa,dmin,nzph)
	CALL intersect_loc(nzph,connec(:,2),c1)
	CALL intersect_loc(nzph,connec(:,3),c2)
	CALL intersect_loc(nzph,connec(:,4),c3)
	CALL intersect_loc(nzph,connec(:,5),c4)
	CALL unique4(c1,c2,c3,c4,ctelem)
	!
	i=1
	DO ielem=1,SIZE(ctelem)
		celem=ctelem(ielem)
		phie=gphi(connec(celem,2:5),1)
		IF (ALL(phie/=0)) THEN
			psie=gpsi(connec(celem,2:5),1)
			IF (MAXVAL(psie)*MINVAL(psie)<1E-6) THEN
				IF (MAXVAL(phie)*MINVAL(phie)<1E-6) THEN
					DO in=1,4
						ctnodes(i)=connec(celem,in+1)
						i=i+1
					END DO
				END IF
			END IF
		END IF
	END DO
	!
	check_ctnodes: DO i=1,nn
		IF (ctnodes(i)==0) THEN
			ALLOCATE(tempct(i-1))
			tempct=ctnodes(1:i-1)
			DEALLOCATE(ctnodes)
			ALLOCATE(ctnodes(i-1))
			ctnodes=tempct
			DEALLOCATE(tempct)
			EXIT check_ctnodes
		END IF
	END DO check_ctnodes
	nodes(ctnodes,4)=444444
	!
	dmin=zero
	compa='ne'
	CALL find_loc(gpsi(:,1),compa,dmin,nzps)
	CALL intersect_loc(nzps,connec(:,2),c1)
	CALL intersect_loc(nzps,connec(:,3),c2)
	CALL intersect_loc(nzps,connec(:,4),c3)
	CALL intersect_loc(nzps,connec(:,5),c4)
	CALL unique4(c1,c2,c3,c4,helem)
	!
	i=1
	ALLOCATE(hnodes(nn))
	hnodes=0
	DO ielem=1,SIZE(helem)
		celem=helem(ielem)
		psie=gpsi(connec(celem,2:5),1)
		IF (ALL(psie/=0)) THEN
			IF (MAXVAL(psie)==1E-6 .OR. MINVAL(psie)==1E-6) THEN
				DO in=1,4
					gn=connec(celem,in+1)
					IF (nodes(gn,4)==0) THEN
						IF (psie(in)==0) THEN
							IF (gphi(gn,1)<=0) THEN
								IF (nodes(gn,2)==0) THEN
									hnodes(i)=gn
									i=i+1
								END IF
							END IF
						END IF
					END IF
				END DO
			ELSE IF (MAXVAL(psie)*MINVAL(psie)<0) THEN
				DO in=1,4
					gn=connec(celem,in+1)
					IF (gphi(gn,1)<=0) THEN
						IF (nodes(gn,4)==0) THEN
							IF (nodes(gn,2)==0) THEN
								hnodes(i)=gn
								i=i+1
							END IF
						END IF
					END IF
				END DO
			END IF
		END IF
	END DO

	check_hnodes: DO i=1,nn
		IF (hnodes(i)==0) THEN
			ALLOCATE(temph(i-1))
			temph=hnodes(1:i-1)
			DEALLOCATE(hnodes)
			ALLOCATE(hnodes(i-1))
			hnodes=temph
			DEALLOCATE(temph)
			EXIT check_hnodes
		END IF
	END DO check_hnodes
	! 
	nn=nn+1+incnode-1
	CALL unique1(ctnodes)
	CALL unique1(bmnodes)
	CALL unique1(hnodes)
	!
	hs: DO i=1,SIZE(hnodes)
		IF (hnodes(i)==0) THEN
			EXIT hs
		END IF
		nodes(hnodes(i),2)=nn
		nn=nn+1
	END DO hs
	!
	ct: DO i=1,SIZE(ctnodes)
		IF (ctnodes(i)==0) THEN
			EXIT ct
		END IF
		nodes(ctnodes(i),4)=nn
		nodes(ctnodes(i),6)=nn+1
		nodes(ctnodes(i),8)=nn+2
		nodes(ctnodes(i),10)=nn+3
		nn=nn+4
	END DO ct
	!
	bm: DO i=1,SIZE(bmnodes)
		IF (bmnodes(i)==0) THEN
			EXIT bm
		END IF
		nodes(bmnodes(i),4)=nn
		nodes(bmnodes(i),6)=nn+1
		nodes(bmnodes(i),8)=nn+2
		nodes(bmnodes(i),10)=nn+3
		nodes(bmnodes(i),12)=nn+4
		nodes(bmnodes(i),14)=nn+5
		nodes(bmnodes(i),16)=nn+6
		nodes(bmnodes(i),18)=nn+7
		nodes(bmnodes(i),20)=nn+8
		nodes(bmnodes(i),22)=nn+9
		nodes(bmnodes(i),24)=nn+10
		nodes(bmnodes(i),26)=nn+11
		nn=nn+12
	END DO bm
	!  
 ELSE
	ALLOCATE(omega(0,0))
 END IF
 ! 
 RETURN
END SUBROUTINE levelset