SUBROUTINE sparse(uenrelem,ii,jj,ss,mm,output)
!
! This subroutine uses vectors ii, jj, and ss to generate an mm-by-mm sparse
! matrix such that output(ii(k),jj(k)) = ss(k). Vectors ii, jj, and ss are all
! the same length. Any elements of ss that are zero are ignored, along with 
! the corresponding values of ii and jj. 
!
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 REAL(iwp),INTENT(IN)::ss(:)
 INTEGER,INTENT(IN)::ii(:),jj(:)
 INTEGER,INTENT(IN)::uenrelem,mm
 REAL(iwp),ALLOCATABLE,INTENT(OUT)::output(:,:)
 INTEGER::k
!
 IF (uenrelem==0) THEN
	ALLOCATE(output(mm,mm))
	output=0.0
 ELSE
	IF (SIZE(ii)/=SIZE(jj).AND.SIZE(jj)/=SIZE(ss)) THEN
		WRITE(*,*)'error: the size of the input arrays much be identical'
		STOP
	ELSE
		ALLOCATE(output(mm,mm))
		output=0.0
		DO k=1,SIZE(ii)
			output(ii(k),jj(k))=output(ii(k),jj(k))+ss(k)
		END DO
	END IF
 END IF
 !
 RETURN
END SUBROUTINE sparse