SUBROUTINE stiff_mat(omega,gchi,zeta,connec,frt,lxe,nxe,nye,prop,nodes,gphi,gpsi,gxyz,&
                    gdof,iter,enrelem,iielem,gstiff)
!
! This function calculates the global stiffness matrix for the discontinuities
! (fracture,inclusion,void) defined by the input parameters.
!
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 REAL(iwp),INTENT(IN)::nodes(:,:),prop(:,:),omega(:,:),gchi(:,:),zeta(:,:)
 REAL(iwp),INTENT(IN)::gphi(:,:),gpsi(:,:),gxyz(:,:),frt(:,:),lxe
 REAL(iwp)::zero=0.0,one=1.0,em,vm,ef,vf,c0,c1,c2,c3,c4,c5,c6,c7,c8
 REAL(iwp)::c(3,3),cm(3,3),cf(3,3),km(8,8),kf(8,8)
 REAL(iwp)::zn(4),pn(4),zp(0),ww,cn(4),detj,zet(4),zab(4)
 REAL(iwp)::nx(4),ny(4),bu(3,8),ba(3,2),bb(3,8),bc(3,24),balpha(3,2)
 REAL(iwp)::x1,x2,x3,x4,y1,y2,y3,y4,xgp,ygp,zgp,psi1,psi2,psi3,psi4,psi
 REAL(iwp)::xx,yy,xloc,yloc,xloc1,yloc1,xloc2,yloc2,rr,theta,zm,za,zmn
 REAL(iwp)::r1,r2,ccs(2),cc,ct,st,sr,st2,ct2,pi=3.141592653589793_iwp
 REAL(iwp)::a1gp,a2gp,a3gp,a4gp,a5gp,a6gp,a7gp,a8gp,a9gp,a10gp,a11gp,a12gp
 REAL(iwp)::a1,a2,a3,a4,b1(3,2),b2(3,2),b3(3,2),b4(3,2),nn(4,31)
 REAL(iwp)::g1,g2,k1,k2,ei,bi,a(12),px(12),py(12),ee,ex,ey,dmin
 REAL(iwp),ALLOCATABLE::sstiff(:,:),allvalt(:),gp(:,:),gw(:,:),gj(:,:)
 REAL(iwp),ALLOCATABLE::xi(:),eta(:),b(:,:),benr(:,:),benrt(:,:)
 REAL(iwp),ALLOCATABLE::xft(:),yft(:),lstiff(:,:),n(:,:),chi(:)
 REAL(iwp),ALLOCATABLE::ppx(:),ppy(:),xyz(:,:),gpn(:,:),gwn(:),gjn(:,:)
 REAL(iwp),ALLOCATABLE,INTENT(OUT)::gstiff(:,:)
 INTEGER,INTENT(IN)::nxe,nye,iter,connec(:,:),enrelem(:),iielem(:)
 INTEGER::nft,gdof,mm,uenrelem,nindext,ielem,n1,n2,n3,n4,ctn,hen,ien,nen
 INTEGER::iloc,row,col,counter,inn,hgp,hi,h
 INTEGER::ni(4),ib(12),ic,i,j,ind
 INTEGER,ALLOCATABLE::allrowt(:),allcolt(:),local(:),localt(:),gr(:)
 CHARACTER(LEN=15)::element
 !
 ALLOCATE(xyz(4,2),local(200))
 xyz=0.0
 local=0
 em=prop(1,1)
 vm=prop(2,1)
 ef=prop(1,2)
 vf=prop(2,2)
 nft=SIZE(gphi,2)
 !
 IF (iter>1) THEN
	gdof=gdof+16*iter
 END IF
 ALLOCATE(gstiff(gdof,gdof))
 gstiff=0
 !
 c1=em*(1.0-vm)/(1.0+vm)/(1.0-2.0*vm)
 c2=em*vm/(1.0+vm)/(1.0-2.0*vm)
 c3=em/2.0/(1.0+vm)
 cm(1,:)=(/c1,c2,zero/)
 cm(2,:)=(/c2,c1,zero/)
 cm(3,:)=(/zero,zero,c3/)
 !
 c1=ef*(1.0-vf)/(1.0+vf)/(1.0-2.0*vf)
 c2=ef*vf/(1.0+vf)/(1.0-2.0*vf)
 c3=ef/2.0/(1.0+vf)
 cf(1,:)=(/c1,c2,zero/)
 cf(2,:)=(/c2,c1,zero/)
 cf(3,:)=(/zero,zero,c3/)
 !
 c0=em/(1.0+vm)/(1.0-2.0*vm)
 c1=c0*(1.0/2.0-2.0*vm/3.0)
 c2=c0*(1.0/8.0)
 c3=c0*(-1.0/4.0+vm/6.0)
 c4=c0*(-1.0/8.0+vm/2.0)
 c5=c0*(-1.0/4.0+vm/3.0)
 c6=c0*(-1.0/8.0)
 c7=c0*(vm/6.0)
 c8=c0*(1.0/8.0-vm/2.0)
 !
 km(1,:)=(/c1,c2,c3,c4,c5,c6,c7,c8/)
 km(2,:)=(/c2,c1,c8,c7,c6,c5,c4,c3/)
 km(3,:)=(/c3,c8,c1,c6,c7,c4,c5,c2/)
 km(4,:)=(/c4,c7,c6,c1,c8,c3,c2,c5/)
 km(5,:)=(/c5,c6,c7,c8,c1,c2,c3,c4/)
 km(6,:)=(/c6,c5,c4,c3,c2,c1,c8,c7/)
 km(7,:)=(/c7,c4,c5,c2,c3,c8,c1,c6/)
 km(8,:)=(/c8,c3,c2,c5,c4,c7,c6,c1/) 
 !
 c0=ef/(1.0+vf)/(1.0-2.0*vf)
 c1=c0*(1.0/2.0-2.0*vf/3.0)
 c2=c0*(1.0/8.0)
 c3=c0*(-1.0/4.0+vf/6.0)
 c4=c0*(-1.0/8.0+vf/2.0)
 c5=c0*(-1.0/4.0+vf/3.0)
 c6=c0*(-1.0/8.0)
 c7=c0*(vf/6.0)
 c8=c0*(1.0/8.0-vf/2.0)
 !
 kf(1,:)=(/c1,c2,c3,c4,c5,c6,c7,c8/)
 kf(2,:)=(/c2,c1,c8,c7,c6,c5,c4,c3/)
 kf(3,:)=(/c3,c8,c1,c6,c7,c4,c5,c2/)
 kf(4,:)=(/c4,c7,c6,c1,c8,c3,c2,c5/)
 kf(5,:)=(/c5,c6,c7,c8,c1,c2,c3,c4/)
 kf(6,:)=(/c6,c5,c4,c3,c2,c1,c8,c7/)
 kf(7,:)=(/c7,c4,c5,c2,c3,c8,c1,c6/)
 kf(8,:)=(/c8,c3,c2,c5,c4,c7,c6,c1/) 
 !
 mm=SIZE(frt,1)
 IF (mm>0) THEN
	IF (nft==1) THEN
		ALLOCATE(xft(1),yft(1))
		xft=frt(mm,1)
		yft=frt(mm,2)
	ELSE IF (nft==2) THEN
		ALLOCATE(xft(2),yft(2))
		xft=(/frt(1,1),frt(mm,1)/)
		yft=(/frt(1,2),frt(mm,2)/)
	END IF
 END IF
 !
 nindext=0
 IF (SIZE(iielem)/=1) THEN
	uenrelem=nxe*nye-SIZE(enrelem)
 ELSE
	uenrelem=nxe*nye-SIZE(enrelem)-SIZE(iielem)
 END IF
 ALLOCATE(allrowt(uenrelem*64),allcolt(uenrelem*64),allvalt(uenrelem*64))
 allrowt=1.0
 allcolt=1.0
 allvalt=0.0
!
 counter=1
 DO ielem=1,nye*nxe
	n1=connec(ielem,2)
	n2=connec(ielem,3)
	n3=connec(ielem,4)
	n4=connec(ielem,5)
	ni=(/n1,n2,n3,n4/)
	nn=nodes(ni,:)	
	cn(1)=gchi(1,n1)
	cn(2)=gchi(1,n2)
	cn(3)=gchi(1,n3)
	cn(4)=gchi(1,n4)
	ctn=0
	hen=0
	ien=0 
	DO i=1,SIZE(nn(:,4))
		IF (nn(i,4)/=0) THEN
			ctn=ctn+1
		ELSE IF (nn(i,2)/=0) THEN
			hen=hen+1
		ELSE IF (nn(i,30)/=0) THEN
			ien=ien+1
		END IF
	END DO
	nen=hen+ctn+ien
	!
	local(1:8)=(/n1*2-1,n1*2,n2*2-1,n2*2,n3*2-1,n3*2,n4*2-1,n4*2/)
	iloc=9
	!
	IF (nen==0) THEN
		ALLOCATE(lstiff(8,8))
		zn=(/zeta(1,n1),zeta(1,n2),zeta(1,n3),zeta(1,n4)/)
		IF (MINVAL(cn)>=0) THEN
			IF (MAXVAL(zn)>=0) THEN
				lstiff=km
			ELSE IF (MAXVAL(zn)<0) THEN
				lstiff=kf
			END IF
		ELSE IF (MAXVAL(cn)>0 .AND. MINVAL(cn)<0) THEN
			x1=gxyz(n1,2)
			x2=gxyz(n2,2)
			x3=gxyz(n3,2)
			x4=gxyz(n4,2)
			y1=gxyz(n1,3)
			y2=gxyz(n2,3)
			y3=gxyz(n3,3)
			y4=gxyz(n4,3)
			xyz(1,:)=(/x1,y1/)
			xyz(2,:)=(/x2,y2/)
			xyz(3,:)=(/x3,y3/)
			xyz(4,:)=(/x4,y4/)
			zn=(/zeta(1,n1),zeta(1,n2),zeta(1,n3),zeta(1,n4)/)
			!
			CALL sub_domain(frt,3,cn,zp,zp,xyz,0,0,gp,gw,gj,ielem)
			ALLOCATE(xi(SIZE(gp,1)),eta(SIZE(gp,1)))
			ALLOCATE(n(SIZE(gp,1),4),chi(SIZE(gp,1)))
			xi=gp(:,1)
			eta=gp(:,2)
			n(:,1)=1.0/4.0*(1.0-xi)*(1.0-eta)
			n(:,2)=1.0/4.0*(1.0+xi)*(1.0-eta)
			n(:,3)=1.0/4.0*(1.0+xi)*(1.0+eta)
			n(:,4)=1.0/4.0*(1.0-xi)*(1.0+eta)
			chi=n(:,1)*cn(1)+n(:,2)*cn(2)+n(:,3)*cn(3)+n(:,4)*cn(4)
			dmin=zero
			compa='le'
			CALL find_loc(chi,compa,dmin,gr)
			CALL rmv_components(gj,gr,gj)
			CALL rmv_components(gp,gr,gp)
			CALL rmv_components(gw,gr,gw)
			DEALLOCATE(xi,eta)
			!
			DO i=1,SIZE(gp,1)
				ALLOCATE(xi(1),eta(1))
				xi=gp(i,1)
				eta=gp(i,2)
				ww=gw(i,1)
				!
				detj=gj(i,1)*gj(i,4)-gj(i,2)*gj(i,3)
				!
				nx=(/2.0/lxe*1.0/4.0*(-1.0+eta),2.0/lxe*1.0/4.0*(1.0-eta),&
					2.0/lxe*1.0/4.0*(1.0+eta),2.0/lxe*1.0/4.0*(-1.0-eta)/)
				ny=(/2.0/lxe*1.0/4.0*(-1.0+xi),2.0/lxe*1.0/4.0*(-1.0-xi),&
					2.0/lxe*1.0/4.0*(1.0+xi),2.0/lxe*1.0/4.0*(1.0-xi)/)
				!
				zgp=n(i,1)*zn(1)+n(i,2)*zn(2)+n(i,3)*zn(3)+n(i,4)*zn(4)
				!
				bu=0
				bu(1,1:7)=(/nx(1),  0  ,nx(2),  0  ,nx(3),  0  ,nx(4)/)
				bu(2,2:8)=(/ny(1),  0  ,ny(2),  0  ,ny(3),  0  ,ny(4)/)
				bu(3,:)=(/ny(1),nx(1),ny(2),nx(2),ny(3),nx(3),ny(4),nx(4)/)
				!
				IF (zgp>0) THEN
				 c=cm
				ELSE
				 c=cf
				END IF
				!
				lstiff=lstiff+ww*MATMUL(TRANSPOSE(bu),MATMUL(c,bu))*detj
				DEALLOCATE(xi,eta)
			END DO	
			DEALLOCATE(n,chi)
		END IF
	ELSE IF (nen>0) THEN
		x1=gxyz(n1,2)
		x2=gxyz(n2,2)
		x3=gxyz(n3,2)
		x4=gxyz(n4,2)
		y1=gxyz(n1,3)
		y2=gxyz(n2,3)
		y3=gxyz(n3,3)
		y4=gxyz(n4,3)
		!
		IF (nen==4) THEN
			xyz(1,:)=(/x1,y1/)
			xyz(2,:)=(/x2,y2/)
			xyz(3,:)=(/x3,y3/)
			xyz(4,:)=(/x4,y4/)
			IF (SIZE(gpsi)==0) THEN
				pn=0
			ELSE
				pn=(/gpsi(n1,1),gpsi(n2,1),gpsi(n3,1),gpsi(n4,1)/)
			END IF
			zn=(/zeta(1,n1),zeta(1,n2),zeta(1,n3),zeta(1,n4)/)
			!
			IF (ien==4) THEN
				CALL sub_domain(frt,3,cn,pn,zn,xyz,0,0,gp,gw,gj,ielem)
			ELSE IF (hen==4) THEN
				CALL sub_domain(frt,3,cn,pn,zn,xyz,1,0,gp,gw,gj,ielem)
			ELSE IF (ctn==4) THEN
				CALL sub_domain(frt,7,cn,pn,zn,xyz,1,0,gp,gw,gj,ielem)
			ELSE
				CALL sub_domain(frt,7,cn,pn,zn,xyz,0,0,gp,gw,gj,ielem)
			END IF
			!
			IF (MINVAL(cn)<0) THEN
				ALLOCATE(xi(SIZE(gp,1)),eta(SIZE(gp,1)))
				ALLOCATE(n(SIZE(gp,1),4),chi(SIZE(gp,1)))
				xi=gp(:,1)
				eta=gp(:,2)
				n(:,1)=1.0/4.0*(1.0-xi)*(1.0-eta)
				n(:,2)=1.0/4.0*(1.0+xi)*(1.0-eta)
				n(:,3)=1.0/4.0*(1.0+xi)*(1.0+eta)
				n(:,4)=1.0/4.0*(1.0-xi)*(1.0+eta)
				chi=n(:,1)*cn(1)+n(:,2)*cn(2)+n(:,3)*cn(3)+n(:,4)*cn(4)
				dmin=zero
				compa='le'
				CALL find_loc(chi,compa,dmin,gr)
				CALL rmv_components(gj,gr,gj)
				CALL rmv_components(gp,gr,gp)
				CALL rmv_components(gw,gr,gw)
				DEALLOCATE(xi,eta,n,chi)
			END IF
			!     
		ELSE
			IF (MINVAL(cn)<0) THEN
				pn=(/gpsi(1,n1),gpsi(1,n2),gpsi(1,n3),gpsi(1,n4)/)
				zn=(/zeta(1,n1),zeta(1,n2),zeta(1,n3),zeta(1,n4)/)
				CALL sub_domain(frt,3,cn,pn,zn,xyz,0,0,gp,gw,gj,ielem)
				ALLOCATE(xi(SIZE(gp,1)),eta(SIZE(gp,1)))
				xi=gp(:,1)
				eta=gp(:,2)
				n(:,1)=1.0/4.0*(1.0-xi)*(1.0-eta)
				n(:,2)=1.0/4.0*(1.0+xi)*(1.0-eta)
				n(:,3)=1.0/4.0*(1.0+xi)*(1.0+eta)
				n(:,4)=1.0/4.0*(1.0-xi)*(1.0+eta)
				chi=n(:,1)*cn(1)+n(:,2)*cn(2)+n(:,3)*cn(3)+n(:,4)*cn(4)
				dmin=zero
				compa='le'
				CALL find_loc(chi,compa,dmin,gr)
				CALL rmv_components(gj,gr,gj)
				CALL rmv_components(gp,gr,gp)
				CALL rmv_components(gw,gr,gw)
				DEALLOCATE(xi,eta,n,chi)
			ELSE
				element='QUAD'
				CALL gauss_pw(6,element,gp,gw)
				IF (counter==1) THEN
					ALLOCATE(gj(0,0))
				ELSE IF (counter==2) THEN
					DEALLOCATE(gj)
					ALLOCATE(gj(0,0))
				END IF
				counter=2
			END IF
		END IF
		!
		DO i=1,SIZE(gp,1)
			ALLOCATE(xi(1),eta(1),n(4,1))
			xi=gp(i,1)
			eta=gp(i,2)
			ww=gw(i,1)
			IF (SIZE(gj)/=0) THEN
				detj=gj(i,1)*gj(i,4)-gj(i,2)*gj(i,3)
			ELSE
				detj=lxe/2.0*lxe/2.0
			END IF
			!
			n(:,1) =(/(1.0-xi)*(1.0-eta)/4.0,(1.0+xi)*(1.0-eta)/4.0,&
					(1.0+xi)*(1.0+eta)/4.0,(1.0-xi)*(1.0+eta)/4.0/)
			nx=(/2.0/lxe*1.0/4.0*(-1.0+eta),2.0/lxe*1.0/4.0*(1.0-eta),&
				2.0/lxe*1.0/4.0*(1.0+eta),2.0/lxe*1.0/4.0*(-1.0-eta)/)
			ny=(/2.0/lxe*1.0/4.0*(-1.0+xi),2.0/lxe*1.0/4.0*(-1.0-xi),&
				2.0/lxe*1.0/4.0*(1.0+xi),2.0/lxe*1.0/4.0*(1.0-xi)/)
			!
			x1=gxyz(n1,2)
			x2=gxyz(n2,2)
			x3=gxyz(n3,2)
			x4=gxyz(n4,2)
			y1=gxyz(n1,3)
			y2=gxyz(n2,3)
			y3=gxyz(n3,3)
			y4=gxyz(n4,3)
			!
			xgp=n(1,1)*x1+n(2,1)*x2+n(3,1)*x3+n(4,1)*x4
			ygp=n(1,1)*y1+n(2,1)*y2+n(3,1)*y3+n(4,1)*y4
			zgp=n(1,1)*zeta(1,n1)+n(2,1)*zeta(1,n2)+n(3,1)*zeta(1,n3)+n(4,1)*zeta(1,n4)
			!
			bu=0
			bu(1,1:7)=(/nx(1),  0  ,nx(2),  0  ,nx(3),  0  ,nx(4)/)
			bu(2,2:8)=(/ny(1),  0  ,ny(2),  0  ,ny(3),  0  ,ny(4)/)
			bu(3,:)=(/ny(1),nx(1),ny(2),nx(2),ny(3),nx(3),ny(4),nx(4)/)
			!
			ind=1
			DO inn=1,4
				IF (nn(inn,2)/=0) THEN
					psi1=gpsi(n1,1)
					psi2=gpsi(n2,1)
					psi3=gpsi(n3,1)
					psi4=gpsi(n4,1)
					psi =n(1,1)*psi1+n(2,1)*psi2+n(3,1)*psi3+n(4,1)*psi4
					!
					hgp=SIGN(one,psi)
					hi =nn(inn,3)
					h  =hgp-hi
					!
					ba=0.0
					ba(1,1)=nx(inn)*h
					ba(2,2)=ny(inn)*h
					ba(3,:)=(/ny(inn)*h,nx(inn)*h/)
					IF (ind==1) THEN
						ALLOCATE(benr(3,2))
						benr=ba
					ELSE
						CALL bmatrix_assemble(benr,ba,benrt)
						DEALLOCATE(benr)
						ALLOCATE(benr(SIZE(benrt,1),SIZE(benrt,2)))
						benr=benrt
						DEALLOCATE(benrt)
					END IF
					ind=ind+1
					!
					IF (i==SIZE(gp,1)) THEN
						local(iloc:(iloc+1))=(/nn(inn,2)*2-1,nn(inn,2)*2/)
						iloc=iloc+2
					END IF
				ELSE IF (nn(inn,4)/=0) THEN
					IF (nft==1) THEN
						xx=xgp-xft(1)
						yy=ygp-yft(1)
						ccs=(/COS(omega(1,1)),SIN(omega(1,1))/)
						xloc=COS(omega(1,1))*xx+SIN(omega(1,1))*yy
						yloc=-SIN(omega(1,1))*xx+COS(omega(1,1))*yy
						rr=SQRT(xloc**2.0+yloc**2.0)
						IF (rr<0.001*lxe) THEN
							rr=0.05*lxe
						END IF
						theta=ATAN2(yloc,xloc)
					ELSE IF (nft==2) THEN
						x1=xgp-xft(1)
						x2=xgp-xft(2)
						y1=ygp-yft(1)
						y2=ygp-yft(2)
						xloc1=COS(omega(1,1))*x1+SIN(omega(1,1))*y1
						yloc1=-SIN(omega(1,1))*x1+COS(omega(1,1))*y1
						xloc2=COS(omega(1,2))*x2+SIN(omega(1,2))*y2
						yloc2=-SIN(omega(1,2))*x2+COS(omega(1,2))*y2
						r1=SQRT(xloc1**2.0+yloc1**2.0)
						r2=SQRT(xloc2**2.0+yloc2**2.0)
						IF (r1>r2) THEN
							rr=r2
							theta=ATAN2(yloc2,xloc2)
							ccs=(/COS(omega(1,2)),SIN(omega(1,2))/)
						ELSE IF (r2>r1) THEN
							rr=r1
							theta=ATAN2(yloc1,xloc1)
							ccs=(/COS(omega(1,1)),SIN(omega(1,1))/)
						END IF
						IF (rr<0.01*lxe) THEN
							rr=0.05*lxe
						END IF
					END IF
					!     
					cc=(2.0*SQRT(rr))**(-1.0)
					ct=ccs(1)
					st=ccs(2)
					!
					IF (nn(inn,12)==0) THEN
						a1gp=SQRT(rr)*SIN(theta/2.0)
						a2gp=SQRT(rr)*COS(theta/2.0)
						a3gp=SQRT(rr)*SIN(theta/2.0)*SIN(theta)
						a4gp=SQRT(rr)*COS(theta/2.0)*SIN(theta)
						!
						a1=a1gp-nn(inn,5)
						a2=a2gp-nn(inn,7)
						a3=a3gp-nn(inn,9)
						a4=a4gp-nn(inn,11)
						!
						ALLOCATE(ppx(4),ppy(4))
						ppx(1)=-SIN(theta/2.0)*ct*cc+COS(theta/2.0)*-st*cc
						ppx(2)= COS(theta/2.0)*ct*cc+SIN(theta/2.0)*-st*cc
						ppx(3)=-SIN(3.0*theta/2.0)*SIN(theta)*ct*cc+&
								(SIN(theta/2.0)+SIN(3.0*theta/2.0)*COS(theta))*-st*cc
						ppx(4)=-COS(3.0*theta/2.0)*SIN(theta)*ct*cc+&
								(COS(theta/2.0)+COS(3.0*theta/2.0)*COS(theta))*-st*cc
						!
						ppy(1)=-SIN(theta/2.0)*st*cc+COS(theta/2.0)*ct*cc
						ppy(2)= COS(theta/2.0)*st*cc+SIN(theta/2.0)*ct*cc
						ppy(3)=-SIN(3.0*theta/2.0)*SIN(theta)*st*cc+&
								(SIN(theta/2.0)+SIN(3.0*theta/2.0)*COS(theta))*ct*cc
						ppy(4)=-COS(3.0*theta/2.0)*SIN(theta)*st*cc+&
								(COS(theta/2.0)+COS(3.0*theta/2.0)*COS(theta))*ct*cc
						!		
						b1=0
						b1(1,1)=nx(inn)*a1+n(inn,1)*ppx(1)
						b1(2,2)=ny(inn)*a1+n(inn,1)*ppy(1)
						b1(3,1:2)=(/ny(inn)*a1+n(inn,1)*ppy(1),nx(inn)*a1+n(inn,1)*ppx(1)/)
						!
						b2=0
						b2(1,1)=nx(inn)*a2+n(inn,1)*ppx(2)
						b2(2,2)=ny(inn)*a2+n(inn,1)*ppy(2)
						b2(3,1:2)=(/ny(inn)*a2+n(inn,1)*ppy(2),nx(inn)*a2+n(inn,1)*ppx(2)/)
						!
						b3=0
						b3(1,1)=nx(inn)*a3+n(inn,1)*ppx(3)
						b3(2,2)=ny(inn)*a3+n(inn,1)*ppy(3)
						b3(3,1:2)=(/ny(inn)*a3+n(inn,1)*ppy(3),nx(inn)*a3+n(inn,1)*ppx(3)/)
						!
						b4=0
						b4(1,1)=nx(inn)*a4+n(inn,1)*ppx(4)
						b4(2,2)=ny(inn)*a4+n(inn,1)*ppy(4)
						b4(3,1:2)=(/ny(inn)*a4+n(inn,1)*ppy(4),nx(inn)*a4+n(inn,1)*ppx(4)/)
						!
						DEALLOCATE(ppx,ppy)
						!
						bb(:,1:2)=b1
						bb(:,3:4)=b2
						bb(:,5:6)=b3
						bb(:,7:8)=b4
						!
						IF (ind==1) THEN
							ALLOCATE(benr(3,8))
							benr=bb
						ELSE						
							CALL bmatrix_assemble(benr,bb,benrt)
							DEALLOCATE(benr)
							ALLOCATE(benr(SIZE(benrt,1),SIZE(benrt,2)))
							benr=benrt
							DEALLOCATE(benrt)
						END IF
						ind=ind+1
						!
						IF (i==SIZE(gp,1)) THEN
							local(iloc:iloc+7)=(/2.0*nn(inn,4)-1.0,2.0*nn(inn,4),&
												2.0*nn(inn,6)-1.0,2.0*nn(inn,6),&
												2.0*nn(inn,8)-1.0,2.0*nn(inn,8),&
												2.0*nn(inn,10)-1.0,2.0*nn(inn,10)/)
							iloc=iloc+8
						END IF
					ELSE
						g1=em/2.0/(1.0+vm)
						g2=ef/2.0/(1.0+vf)
						k1=3.0-4.0*vm
						k2=3.0-4.0*vf
						!
						bi=(g1*(k2-1.0)-g2*(k1-1.0))/(g1*(k2+1.0)+g2*(k1+1.0))
						ei=1.0/(2.0*pi)*log((1.0-bi)/(1.0+bi))
						!
						sr=SQRT(rr)
						st=SIN(theta)
						st2=SIN(theta/2.0)
						ct2=COS(theta/2.0)
						!
						a1gp  = sr*COS(3.0*LOG(rr))*EXP(-ei*theta)*st2
						a2gp  = sr*COS(3.0*LOG(rr))*EXP(-ei*theta)*ct2
						a3gp  = sr*COS(3.0*LOG(rr))*EXP(ei*theta)*st2
						a4gp  = sr*COS(3.0*LOG(rr))*EXP(ei*theta)*ct2
						a5gp  = sr*COS(3.0*LOG(rr))*EXP(ei*theta)*st2*st
						a6gp  = sr*COS(3.0*LOG(rr))*EXP(ei*theta)*ct2*st
						a7gp  = sr*SIN(3.0*LOG(rr))*EXP(-ei*theta)*st2
						a8gp  = sr*SIN(3.0*LOG(rr))*EXP(-ei*theta)*ct2
						a9gp  = sr*SIN(3.0*LOG(rr))*EXP(ei*theta)*st2
						a10gp = sr*SIN(3.0*LOG(rr))*EXP(ei*theta)*ct2
						a11gp = sr*SIN(3.0*LOG(rr))*EXP(ei*theta)*st2*st
						a12gp = sr*SIN(3.0*LOG(rr))*EXP(ei*theta)*ct2*st
						!
						a=(/a1gp-nn(inn, 5), a2gp-nn(inn, 7), a3gp-nn(inn, 9), a4gp-nn(inn,11),&
							a5gp-nn(inn,13), a6gp-nn(inn,15), a7gp-nn(inn,17), a8gp-nn(inn,19),&
							a9gp-nn(inn,21),a10gp-nn(inn,23),a11gp-nn(inn,25),a12gp-nn(inn,27)/)
						!
						px=(/(-EXP(-ei*theta)*SIN(theta/2.0)*(COS(ei*LOG(rr))+&
								2.0*ei*SIN(ei*LOG(rr)-theta)))*cc,&
							(EXP(-ei*theta)*COS(theta/2.0)*(COS(ei*LOG(rr))-&
								2.0*ei*SIN(ei*LOG(rr)-theta)))*cc,&
							(-EXP(ei*theta)*SIN(theta/2.0)*(COS(ei*LOG(rr))+&
								2.0*ei*SIN(ei*LOG(rr)+theta)))*cc,&
							(EXP(ei*theta)*COS(theta/2.0)*(COS(ei*LOG(rr))-&
								2.0*ei*SIN(ei*LOG(rr)+theta)))*cc,&
							(-EXP(ei*theta)*SIN(theta)*(COS(ei*LOG(rr))*SIN(3.0*theta/2.0)+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*SIN(theta/2.0)))*cc,&
							(-EXP(ei*theta)*SIN(theta)*(COS(ei*LOG(rr))*COS(3.0*theta/2.0)+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*COS(theta/2.0)))*cc,&
							(EXP(-ei*theta)*SIN(theta/2.0)*(-SIN(ei*LOG(rr))+&
								2.0*ei*COS(ei*LOG(rr)-theta)))*cc,&
							(EXP(-ei*theta)*COS(theta/2.0)*(SIN(ei*LOG(rr))+&
								2.0*ei*COS(ei*LOG(rr)-theta)))*cc,&
							(EXP(ei*theta)*SIN(theta/2.0)*(-SIN(ei*LOG(rr))+&
								2.0*ei*COS(ei*LOG(rr)+theta)))*cc,&
							(EXP(ei*theta)*COS(theta/2.0)*(SIN(ei*LOG(rr))+&
								2.0*ei*COS(ei*LOG(rr)+theta)))*cc,&
							(EXP(ei*theta)*SIN(theta)*(-SIN(ei*LOG(rr))*SIN(3.0*theta/2.0)+&
								2.0*ei*COS(ei*LOG(rr)+theta)*SIN(theta/2.0)))*cc,&
							(EXP(ei*theta)*SIN(theta)*(-SIN(ei*LOG(rr))*COS(3.0*theta/2.0)+&
								2.0*ei*COS(ei*LOG(rr)+theta)*COS(theta/2.0)))*cc/)
						!
						py=(/(EXP(-ei*theta)*(COS(ei*LOG(rr))*COS(theta/2.0)-&
								2.0*ei*COS(ei*LOG(rr)-theta)*SIN(theta/2.0)))*cc,&
							(EXP(-ei*theta)*(COS(ei*LOG(rr))*SIN(theta/2.0)-&
								2.0*ei*COS(ei*LOG(rr)-theta)*COS(theta/2.0)))*cc,&
							(EXP(ei*theta)*(COS(ei*LOG(rr))*COS(theta/2.0)+&
								2.0*ei*COS(ei*LOG(rr)+theta)*SIN(theta/2.0)))*cc,&
							(EXP(ei*theta)*(COS(ei*LOG(rr))*SIN(theta/2.0)+&
								2.0*ei*COS(ei*LOG(rr)+theta)*COS(theta/2.0)))*cc,&
							(EXP(ei*theta)*(COS(ei*LOG(rr))*(SIN(theta/2.0)+SIN(3*theta/2)*COS(theta))+&
								2.0*ei*COS(ei*LOG(rr)+theta)*SIN(theta/2.0)*SIN(theta)))*cc,&
							(EXP(ei*theta)*(COS(ei*LOG(rr))*(COS(theta/2.0)+COS(3*theta/2)*COS(theta))+&
								2.0*ei*COS(ei*LOG(rr)+theta)*COS(theta/2.0)*SIN(theta)))*cc,&
							(EXP(-ei*theta)*(SIN(ei*LOG(rr))*COS(theta/2.0)-&
								2.0*ei*SIN(ei*LOG(rr)-theta)*SIN(theta/2.0)))*cc,&
							(EXP(-ei*theta)*(SIN(ei*LOG(rr))*SIN(theta/2.0)-&
								2.0*ei*SIN(ei*LOG(rr)-theta)*COS(theta/2.0)))*cc,&
							(EXP(ei*theta)*(SIN(ei*LOG(rr))*COS(theta/2.0)+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*SIN(theta/2.0)))*cc,&
							(EXP(ei*theta)*(SIN(ei*LOG(rr))*SIN(theta/2.0)+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*COS(theta/2.0)))*cc,&
							(EXP(ei*theta)*(SIN(ei*LOG(rr))*(SIN(theta/2.0)+SIN(3.0*theta/2.0)*COS(theta))+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*SIN(theta/2.0)*SIN(theta)))*cc,&
							(EXP(ei*theta)*(SIN(ei*LOG(rr))*(COS(theta/2.0)+COS(3.0*theta/2.0)*COS(theta))+&
								2.0*ei*SIN(ei*LOG(rr)+theta)*COS(theta/2.0)*SIN(theta)))*cc/)
						!
						ALLOCATE(ppx(12),ppy(12))
						ppx=(/px(1)* ct+py(1)* -st,px(2)* ct+py(2)* -st,px(3)* ct+py(3)* -st,&
							px(4)* ct+py(4)* -st,px(5)* ct+py(5)* -st,px(6)* ct+py(6)* -st,&
							px(7)* ct+py(7)* -st,px(8)* ct+py(8)* -st,px(9)* ct+py(9)* -st,&
							px(10)*ct+py(10)*-st,px(11)*ct+py(11)*-st,px(12)*ct+py(12)*-st/)
						!
						ppy=(/px(1)* st+py(1)* ct,px(2)* st+py(2)* ct,px(3)* st+py(3)* ct,&
							px(4)* st+py(4)* ct,px(5)* st+py(5)* ct,px(6)* st+py(6)* ct,&
							px(7)* st+py(7)* ct,px(8)* st+py(8)* ct,px(9)* st+py(9)* ct,&
							px(10)*st+py(10)*ct,px(11)*st+py(11)*ct,px(12)*st+py(12)*ct/)
						!
						bc=0.0
						balpha=0.0
						DO ic=1,12
							balpha(1,1)=nx(inn)*a(ic)+n(inn,1)*ppx(ic)
							balpha(2,2)=nx(inn)*a(ic)+n(inn,1)*ppy(ic)
							balpha(3,:)=(/nx(inn)*a(ic)+n(inn,1)*ppy(ic),&
										nx(inn)*a(ic)+n(inn,1)*ppx(ic)/)
							bc(:,2*ic-1:2*ic)=balpha
						END DO
						!
						IF (ind==1) THEN
							ALLOCATE(benr(3,24))
							benr=bc
						ELSE						
							CALL bmatrix_assemble(benr,bc,benrt)
							DEALLOCATE(benr)
							ALLOCATE(benr(SIZE(benrt,1),SIZE(benrt,2)))
							benr=benrt
							DEALLOCATE(benrt)
						END IF
						ind=ind+1
						!
						DEALLOCATE(ppx,ppy)
						IF (i==SIZE(gp,1)) THEN
							ib=(/4,6,8,10,12,14,16,18,20,22,24,26/)
							DO j=1,SIZE(ib)
								local(iloc:iloc+1)=(/2*nn(inn,ib(j))-1,2*nn(inn,ib(j))/)
								iloc=iloc+2
							END DO
						END IF
					END IF
				END IF
				!
				IF (nn(inn,30)/=0) THEN
					zet(1)=zeta(1,n1)
					zet(2)=zeta(1,n2)
					zet(3)=zeta(1,n3)
					zet(4)=zeta(1,n4)
					zab=ABS(zet)
					zm =n(1,1)*zet(1)+n(2,1)*zet(2)+n(3,1)*zet(3)+n(4,1)*zet(4)
					za =n(1,1)*zab(1)+n(2,1)*zab(2)+n(3,1)*zab(3)+n(4,1)*zab(4)
					zmn=zm/ABS(zm)
					!
					ee=za-ABS(zm)
					ex=nx(1)*zab(1)+nx(2)*zab(2)+nx(3)*zab(3)+nx(4)*zab(4)-&
						zmn*(nx(1)*zet(1)+nx(2)*zet(2)+nx(3)*zet(3)+nx(4)*zet(4))
					ey=ny(1)*zab(1)+ny(2)*zab(2)+ny(3)*zab(3)+ny(4)*zab(4)-&
						zmn*(ny(1)*zet(1)+ny(2)*zet(2)+ny(3)*zet(3)+ny(4)*zet(4))
					ba=0
					ba(1,1)=nx(inn)*ee+n(inn,1)*ex
					ba(2,2)=ny(inn)*ee+n(inn,1)*ey
					ba(3,:)=(/ny(inn)*ee+n(inn,1)*ey,nx(inn)*ee+n(inn,1)*ex/)
					!
					CALL bmatrix_assemble(benr,ba,benrt)
					DEALLOCATE(benr)
					ALLOCATE(benr(SIZE(benrt,1),SIZE(benrt,2)))
					benr=benrt
					DEALLOCATE(benrt)
					!
					IF (i==SIZE(gp,1)) THEN
						local(iloc:iloc+1)=(/2*nn(inn,30)-1,2*nn(inn,30)/)
						iloc=iloc+2
					END IF
				END IF
			END DO  
			! 
			IF (zgp>0) THEN
				c=cm
			ELSE
				c=cf
			END IF
			!
			CALL bmatrix_assemble(bu,benr,b)
			IF (i==1) THEN			
				ALLOCATE(lstiff(SIZE(b,2),SIZE(b,2)))
				lstiff=0
			END IF
			lstiff=lstiff+ww*MATMUL(TRANSPOSE(b),MATMUL(c,b))*detj
			DEALLOCATE(benr) 
			DEALLOCATE(xi,eta,n)
		END DO
	END IF
	IF (SIZE(lstiff,1)==8) THEN
		DO row=1,8
			DO col=1,8
				nindext=nindext+1
				allrowt(nindext)=local(row)
				allcolt(nindext)=local(col)
				allvalt(nindext)=lstiff(row,col)
			END DO
		END DO
	ELSE
		ALLOCATE(localt(iloc-1))
		localt=local(1:iloc-1)
		gstiff(localt,localt)=gstiff(localt,localt)+lstiff
		DEALLOCATE(localt)
	END IF
	DEALLOCATE(lstiff)
 END DO
 !
 CALL sparse(uenrelem,allrowt,allcolt,allvalt,gdof,sstiff)
 gstiff=gstiff+sstiff
 !    
RETURN
END SUBROUTINE stiff_mat