SUBROUTINE sub_domain(frt,npt,chi,psi,zeta,xyz,ctflag,jiflag,gp,gw,gj,add)
!
! This function subdivides enriched elements and determines the gauss 
! points and weights to be used in the integration during the assembly of 
! the stiffness matrix.
!
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 REAL(iwp),INTENT(IN)::frt(:,:),chi(:),psi(:),zeta(:)
 REAL(iwp),ALLOCATABLE,INTENT(IN)::xyz(:,:)
 REAL(iwp),ALLOCATABLE::nxyz(:,:),rxyz(:,:),xyzp(:,:),qq(:,:),ww(:,:),xyzl(:,:)
 REAL(iwp),ALLOCATABLE,INTENT(OUT)::gp(:,:),gw(:,:),gj(:,:)
 REAL(iwp)::xctip(2),yctip(2),dd(2),xyct(1,2),nn(4),xypnt(1,2)
 REAL(iwp)::xpnt,ypnt,xin,etn,xct,yct,rr
 REAL(iwp)::xmn,xmx,xmd,ymn,ymx,ymd,dmin,le,xm,ym,xd,yd
 INTEGER,INTENT(IN)::ctflag,jiflag,npt,add
 INTEGER,ALLOCATABLE::node(:,:),nnode(:,:),rnode(:,:),coord(:,:),tri(:,:)
 INTEGER::k,mm,indi,corner(5),ntip(1,2),n1,n2,pnt(1,2),xi,eta
 INTEGER::mj,ind,counter
 CHARACTER(LEN=15)::element
 !
 ALLOCATE(nxyz(SIZE(xyz,1),SIZE(xyz,2)))
 nxyz=xyz
 xct=0
 yct=0
 !
 indi=1
 IF (ctflag==1) THEN
	mm=SIZE(frt,1)
	!
	xctip=(/frt(mm,1),frt(1,1)/)
	yctip=(/frt(mm,2),frt(1,2)/)
	!
	xmn=MINVAL(nxyz(:,1))
	xmx=MAXVAL(nxyz(:,1))
	xmd=(xmn+xmx)/2.0
	ymn=MINVAL(nxyz(:,2))
	ymx=MAXVAL(nxyz(:,2))
	ymd=(ymn+ymx)/2.0
	!
	check: DO i=1,SIZE(xctip)
		IF (xctip(i)>=xmn .AND. xctip(i)<=xmx) THEN
			IF (yctip(i)>=ymn .AND. yctip(i)<=ymx) THEN
				xct=xctip(i)
				yct=yctip(i)
				xyct(1,:)=(/xct,yct/)
				indi=2
				EXIT check
			END IF
		END IF
	END DO check
	!
	IF (indi==1) THEN
		DO i=1,SIZE(xctip)
			dd(i)=SQRT((xctip(i)-xmd)**2+(yctip(i)-ymd)**2)
		END DO
		ind=MINLOC(dd,1)
		xct=xctip(ind)
		yct=yctip(ind)
		xyct(1,:)=(/xct,yct/)
	END IF
	!
	CALL reallocate_real(xyz,xyct,nxyz)
	!
	le=nxyz(2,1)-nxyz(1,1)
	xm=(nxyz(2,1)+nxyz(1,1))/2.0
	ym=(nxyz(3,2)+nxyz(2,2))/2.0
	xd=xct-xm
	yd=yct-ym
	xi=2.0*xd/le
	eta=2.0*yd/le
	ntip(1,:)=(/xi,eta/)
 END IF
 !
 ALLOCATE(node(4,2))
 corner=(/1,2,3,4,1/)
 node(1,:)=(/-1,-1/)
 node(2,:)=(/ 1,-1/)
 node(3,:)=(/ 1, 1/)
 node(4,:)=(/-1, 1/)
 IF (ctflag==1) THEN
	CALL reallocate_int(node,ntip,nnode)
 ELSE
	ALLOCATE(nnode(SIZE(node,1),SIZE(node,2)))
	nnode=node
 END IF
 !
 IF (SIZE(psi)==4) THEN
	counter=1
	DO i=1,4
		n1=corner(i)
		n2=corner(i+1)
		IF (psi(n1)*psi(n2)<0) THEN
			rr=psi(n1)/(psi(n1)-psi(n2))
			IF (counter==1 .OR. counter==3) THEN
				pnt(1,:)=(1.0-rr)*nnode(n1,:)+rr*nnode(n2,:)
			ELSE IF (counter==2 .OR. counter==4) THEN
				pnt(1,:)=(1.0-rr)*rnode(n1,:)+rr*rnode(n2,:)
			END IF
			xi=pnt(1,1)
			eta=pnt(1,2)
			IF (counter==1 .OR. counter==3) THEN
				xpnt=((1.0-xi)*(1.0-eta)*nxyz(1,1)+(1.0+xi)*(1.0-eta)*nxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,1)+(1.0-xi)*(1.0+eta)*nxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*nxyz(1,2)+(1.0+xi)*(1.0-eta)*nxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,2)+(1.0-xi)*(1.0+eta)*nxyz(4,2))/4.0
			ELSE IF (counter==2 .OR. counter==4) THEN
				xpnt=((1.0-xi)*(1.0-eta)*rxyz(1,1)+(1.0+xi)*(1.0-eta)*rxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,1)+(1.0-xi)*(1.0+eta)*rxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*rxyz(1,2)+(1.0+xi)*(1.0-eta)*rxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,2)+(1.0-xi)*(1.0+eta)*rxyz(4,2))/4.0
			END IF
			xypnt(1,1)=xpnt
			xypnt(1,2)=ypnt
			IF (counter==1 .OR. counter==3) THEN
				CALL reallocate_real(nxyz,xypnt,rxyz)
				CALL reallocate_int(nnode,pnt,rnode)
			ELSE IF (counter==2 .OR. counter==4) THEN
				CALL reallocate_real(rxyz,xypnt,nxyz)
				CALL reallocate_int(rnode,pnt,nnode)
			END IF
			counter=counter+1
		END IF
	END DO
 END IF
 IF (counter==2 .OR. counter==4) THEN
	DEALLOCATE(nxyz,nnode)
	ALLOCATE(nxyz(SIZE(rxyz,1),SIZE(rxyz,2)))
	ALLOCATE(nnode(SIZE(rnode,1),SIZE(rnode,2)))
	nxyz=rxyz
	nnode=rnode
 END IF
 !
 IF (SIZE(zeta)==4) THEN
	counter=1
	DO i=1,4
		n1=corner(i)
		n2=corner(i+1)
		IF (zeta(n1)*zeta(n2)<0) THEN
			rr=zeta(n1)/(zeta(n1)-zeta(n2))
			IF (counter==1 .OR. counter==3) THEN
				pnt(1,:)=(1.0-rr)*nnode(n1,:)+rr*nnode(n2,:)
			ELSE IF (counter==2 .OR. counter==4) THEN
				pnt(1,:)=(1.0-rr)*rnode(n1,:)+rr*rnode(n2,:)
			END IF
			xi=pnt(1,1)
			eta=pnt(1,2)
			IF (counter==1 .OR. counter==3) THEN
				xpnt=((1.0-xi)*(1.0-eta)*nxyz(1,1)+(1.0+xi)*(1.0-eta)*nxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,1)+(1.0-xi)*(1.0+eta)*nxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*nxyz(1,2)+(1.0+xi)*(1.0-eta)*nxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,2)+(1.0-xi)*(1.0+eta)*nxyz(4,2))/4.0
			ELSE IF (counter==2 .OR. counter==4) THEN
				xpnt=((1.0-xi)*(1.0-eta)*rxyz(1,1)+(1.0+xi)*(1.0-eta)*rxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,1)+(1.0-xi)*(1.0+eta)*rxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*rxyz(1,2)+(1.0+xi)*(1.0-eta)*rxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,2)+(1.0-xi)*(1.0+eta)*rxyz(4,2))/4.0
			END IF
			xypnt(1,1)=xpnt
			xypnt(1,2)=ypnt
			IF (counter==1 .OR. counter==3) THEN
				CALL reallocate_real(nxyz,xypnt,rxyz)
				CALL reallocate_int(nnode,pnt,rnode)
			ELSE IF (counter==2 .OR. counter==4) THEN
				CALL reallocate_real(rxyz,xypnt,nxyz)
				CALL reallocate_int(rnode,pnt,nnode)
			END IF
			counter=counter+1
		END IF
	END DO
 END IF
 IF (counter==2 .OR. counter==4) THEN
	DEALLOCATE(nxyz,nnode)
	ALLOCATE(nxyz(SIZE(rxyz,1),SIZE(rxyz,2)))
	ALLOCATE(nnode(SIZE(rnode,1),SIZE(rnode,2)))
	nxyz=rxyz
	nnode=rnode
 END IF
 !
 IF (SIZE(chi)==4) THEN
	counter=1
	DO i=1,4
		n1=corner(i)
		n2=corner(i+1)
		IF (chi(n1)*chi(n2)<0) THEN
			rr=chi(n1)/(chi(n1)-chi(n2))
			IF (counter==1 .OR. counter==3) THEN
				pnt(1,:)=(1.0-rr)*nnode(n1,:)+rr*nnode(n2,:)
			ELSE IF (counter==2 .OR. counter==4) THEN
				pnt(1,:)=(1.0-rr)*rnode(n1,:)+rr*rnode(n2,:)
			END IF
			xi=pnt(1,1)
			eta=pnt(1,2)
			IF (counter==1 .OR. counter==3) THEN
				xpnt=((1.0-xi)*(1.0-eta)*nxyz(1,1)+(1.0+xi)*(1.0-eta)*nxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,1)+(1.0-xi)*(1.0+eta)*nxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*nxyz(1,2)+(1.0+xi)*(1.0-eta)*nxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*nxyz(3,2)+(1.0-xi)*(1.0+eta)*nxyz(4,2))/4.0
			ELSE IF (counter==2 .OR. counter==4) THEN
				xpnt=((1.0-xi)*(1.0-eta)*rxyz(1,1)+(1.0+xi)*(1.0-eta)*rxyz(2,1)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,1)+(1.0-xi)*(1.0+eta)*rxyz(4,1))/4.0
				ypnt=((1.0-xi)*(1.0-eta)*rxyz(1,2)+(1.0+xi)*(1.0-eta)*rxyz(2,2)+&
					(1.0+xi)*(1.0+eta)*rxyz(3,2)+(1.0-xi)*(1.0+eta)*rxyz(4,2))/4.0
			END IF
			xypnt(1,1)=xpnt
			xypnt(1,2)=ypnt
			IF (counter==1 .OR. counter==3) THEN
				CALL reallocate_real(nxyz,xypnt,rxyz)
				CALL reallocate_int(nnode,pnt,rnode)
			ELSE IF (counter==2 .OR. counter==4) THEN
				CALL reallocate_real(rxyz,xypnt,nxyz)
				CALL reallocate_int(rnode,pnt,nnode)
			END IF	
			counter=counter+1
		END IF
	END DO
 END IF
 IF (counter==2 .OR. counter==4) THEN
	DEALLOCATE(nxyz,nnode)
	ALLOCATE(nxyz(SIZE(rxyz,1),SIZE(rxyz,2)))
	ALLOCATE(nnode(SIZE(rnode,1),SIZE(rnode,2)))
	nxyz=rxyz
	nnode=rnode
 END IF
 !
 IF (add==172) THEN
	ALLOCATE(tri(4,3))
	tri(1,:)=(/4,5,6/)
	tri(2,:)=(/5,1,2/)
	tri(3,:)=(/4,6,3/)
	tri(4,:)=(/5,2,6/)
 ELSE IF (add==173 .OR. add==174 .OR. add==175 .OR. add==176) THEN
	ALLOCATE(tri(6,3))
	tri(1,:)=(/4,5,7/)
	tri(2,:)=(/5,1,7/)
	tri(3,:)=(/6,4,7/)
	tri(4,:)=(/1,2,7/)
	tri(5,:)=(/4,6,3/)
	tri(6,:)=(/7,2,6/)
 ELSE IF (add==177 .OR. add==178 .OR. add==179) THEN
	ALLOCATE(tri(6,3))
	tri(1,:)=(/4,7,6/)
	tri(2,:)=(/7,1,2/)
	tri(3,:)=(/7,2,6/)
	tri(4,:)=(/6,2,5/)
	tri(5,:)=(/3,6,5/)
	tri(6,:)=(/3,4,6/)
 ELSE IF (add==180) THEN
	ALLOCATE(tri(4,3))
	tri(1,:)=(/4,6,5/)
	tri(2,:)=(/6,1,2/)
	tri(3,:)=(/4,5,3/)
	tri(4,:)=(/6,2,5/)
 ELSE IF (add==181) THEN
	ALLOCATE(tri(6,3))
	tri(1,:)=(/5,7,1/)
	tri(2,:)=(/5,1,2/)
	tri(3,:)=(/3,4,5/)
	tri(4,:)=(/3,5,6/)
	tri(5,:)=(/4,7,5/)
	tri(6,:)=(/5,2,6/)
 END IF
 !
 element='TRI'
 CALL gauss_pw(npt,element,qq,ww)
 !
 indi=1
 ALLOCATE(coord(3,2),xyzl(3,2))
 ALLOCATE(gp(SIZE(tri,1)*SIZE(ww),2))
 ALLOCATE(gw(SIZE(tri,1)*SIZE(ww),1))
 ALLOCATE(gj(SIZE(tri,1)*SIZE(ww),4))
 DO k=1,SIZE(tri,1)
	coord=nnode(tri(k,:),:)
	xyzl=nxyz(tri(k,:),:)
	DO i=1,SIZE(ww)
		xin=qq(i,1)
		etn=qq(i,2)
		gp(indi,1)=(1.0-xin-etn)*coord(1,1)+(xin)*coord(2,1)+(etn)*coord(3,1)
		gp(indi,2)=(1.0-xin-etn)*coord(1,2)+(xin)*coord(2,2)+(etn)*coord(3,2)
		gw(indi,1)=ww(i,1)/2.0
		gj(indi,1)=-xyzl(1,1)+xyzl(2,1)
		gj(indi,2)=-xyzl(1,2)+xyzl(2,2)
		gj(indi,3)=-xyzl(1,1)+xyzl(3,1)
		gj(indi,4)=-xyzl(1,2)+xyzl(3,2)
		indi=indi+1
	END DO
 END DO
 !
 RETURN
END SUBROUTINE sub_domain